# Installing Gnosis_Utils:
#
#     python setup.py build
#     python setup.py install
#
# [FYI, this file is called 'setup.py' because that
#  is what users traditionally expect, but also because
#  that name is hardcoded a few places in distutils.]
#

from distutils.core import setup
import os, sys, glob, re

# sanity check for botched Python installations
# (probably means that pyexpat wasn't built)
def is_DOM_okay():
    s = '<?xml version="1.0"?><P></P>'
    import StringIO
    try:
        from xml.dom import minidom
        minidom.parse(StringIO.StringIO(s))
        return 1
    except:
        return 0

if not os.path.exists('MANIFEST'):
    # should only happen in the dev environment if you do a 'make clean'
    # without remaking the sdist
    print "AAARGH! Missing MANIFEST - you probably need to do 'make dist' first"
    sys.exit(1)

# ensure that an entire path exists
# os.makedirs doesn't seem to work as I expected ... :-(
# [XXX I think this was a bug in earlier Python versions.
#      Need to try again under Python 2.3+.  --fpm]
def makepath(path):
    # path is from MANIFEST, so we know it'll have forward slashes
    # (see distutils docs)
    parts = path.split('/')
    partial = ''
    for part in parts:
        partial = os.path.join(partial,part)
        try: os.mkdir(partial)
        except: pass

# try running setup(), returning 1 on success, 0 on failure
def do_setup():
    try:
        kwargs = {
            # Note: PKG-INFO is autogenerated & placed in the sdist
            #       using info from this dictionary, as follows:
            #
            #   name             -> Name
            #   version          -> Version
            #   description      -> Summary
            #   url              -> Home-page
            #   author           -> Author
            #   author_email     -> Author-email
            #   license          -> License
            #   long_description -> Description
            #   platforms        -> Platform (multiple lines if needed)

            'name' : "Gnosis_Utils",
            'version': "1.1.1",
            'description': "Modules and Utilities for XML Documents and Other Miscellany",
            'long_description': "Include modules: xml.pickle, xml.objectify, xml.indexer, xml.validity, and friends.",
            'long_description':  "Include modules:  xml.pickle, xml.objectify, "\
            
                                 "indexer, xml.indexer, xml.validity, gnosis.anon, "\
                                 "multimethods, magic, and friends.",
            
            'author': "Gnosis Software",
            'author_email': "mertz@gnosis.cx",
            'url': "http://gnosis.cx/download/",
            'packages': ["gnosis",
                         "gnosis.anon",
                         "gnosis.magic",
                         "gnosis.util",
                         "gnosis.util.convert",
                         "gnosis.xml",
                         "gnosis.xml.objectify",
                         "gnosis.xml.objectify.doc",
                         "gnosis.xml.objectify.test",
                         "gnosis.xml.pickle",
                         "gnosis.xml.pickle.ext",
                         "gnosis.xml.pickle.util",
                         "gnosis.xml.pickle.doc",
                         "gnosis.xml.pickle.test",
                         "gnosis.xml.pickle.parsers",
                         "gnosis.xml.relax",
                         "gnosis.xml.validity",
                         ],
            'license': "Public Domain"
            }


        # Python 2.0 dies if you pass a 'platforms' arg, but if we
        # leave it out, we get UNKNOWN in PKG-INFO. Our sdist will
        # always be built with Python >= 2.1, so it's not a problem.
        # This keeps us from bombing when installing on Python 2.0.
        if int(sys.version_info[0]) >= 2 and \
           int(sys.version_info[1]) >= 1:
            kwargs['platforms'] = 'Any'

        # Python <= 2.2 misspells 'license' (deprecation warning in 2.3+)
        if not (int(sys.version_info[0]) >= 2 and \
                int(sys.version_info[1]) >= 3):
            kwargs['licence'] = kwargs['license']
            del kwargs['license']
            
        apply(setup,[],kwargs)
        return 1
    except:
        return 0

# by default, we copy EVERYTHING from MANIFEST to build/lib*.
# this has the nice side-effect of converting all text files
# to the platform text format. setting this to zero means we
# only copy files NOT already under build/lib*
copy_all_files = 1

def copy_extra_files():
    destroot = glob.glob(os.path.join('build','lib'))[0]

    # go through MANIFEST to see what is supposed to be under build directory
    print "Copying extra files to %s ..." % destroot

    f = open('MANIFEST','r')
    for srcfile in f.readlines():
        srcfile = srcfile.rstrip() # remove newline char(s)
        dest = os.path.join(destroot,os.path.normpath(srcfile))
        if not os.path.exists(dest) or copy_all_files:
            dir,file = os.path.split(dest)
            if file == 'setup.py':
                continue # skip
            makepath(dir)
            open(dest,'w').write(open(srcfile,'r').read())

if 'master' in sys.argv:
    # make kinda sure I'm in the right place
    files = ['Makefile','gnosis','MANIFEST']
    for file in files:
        if not os.path.exists(file):
            print "*** Hey, you're supposed to run this from the toplevel."
            sys.exit(1)

    # find version number in setup.py
    f = open('setup.py')
    for line in f.xreadlines():
        m = re.match(r"\s*'version'\s*:\s*\"(.+)\"",line)
        if m:
            myver = m.group(1)

    # ensure the working tree is named 'Gnosis_Utils-NNN-master'
    # and tar it up
    savedir = os.getcwd()
    mydir = os.path.split(os.getcwd())[1]

    curname = os.path.split(os.getcwd())[1]
    newname = 'Gnosis_Utils-%s-master' % myver

    if not os.path.isdir('dist'):
        os.mkdir('dist')
        
    os.chdir('..')

    if curname != newname:
        if os.path.exists(newname):
            print "*** Yikes - %s already exists - bailing out!!!"
            sys.exit(1)

        print "******************************************************************"      
        print "*** NOTE - About to rename:\n***  %s/%s\n to:\n*** %s/%s\n*** Press ENTER to continue, or Ctrl-C to abort ..." % (os.getcwd(),curname,os.getcwd(),newname)
        print "******************************************************************"
        sys.stdin.readline()
        
        print "RENAMING %s to %s" % (curname,newname)
        os.rename(curname, newname)

    os.system('rm -f %s.tar.gz' % newname)
    os.system('tar cf %s.tar %s' % (newname,newname))
    os.system('gzip -9 %s.tar' % newname)

    os.system('mv %s.tar.gz %s/dist' % (newname,newname))
    os.chdir(newname)
    
    sys.exit(0)
    
if 'install' in sys.argv:
    if not os.path.isdir('build'):
        print "** Please run build command first **"
        sys.exit(1)
    copy_extra_files()

if do_setup() != 1:
    print "** ERROR: setup() failed"
    sys.exit(1)

if not is_DOM_okay():
    print ""
    print "** WARNING: xml.dom.minidom is not working."
    print "**       Some portions of the package will not work."
    print "**       If you are using Python 2.0, you should install PyXML:"
    print "**           http://pyxml.sourceforge.net/topics/download.html"
    print ""

